
# q-expansion of newform 3750.2.c.e, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3750
weight = 2
poly_data = [1, 0, -1, 0, 1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 0, 0, 1, 0, 0], 1], [[0, 0, 0, 0, 1, 0, 1, 0], 1], [[0, 0, 0, 1, 0, 0, 0, 1], 1], [[0, 0, 0, 1, 0, 0, 0, -1], 1], [[0, 0, 0, 0, 1, 0, -1, 0], 1], [[0, 2, 0, -1, 0, 1, 0, -1], 1], [[-1, 0, 2, 0, -1, 0, 1, 0], 1]]

hecke_ring_character_values = [[2501, [1, 0, 0, 0, 0, 0, 0, 0]], [3127, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 1, -1, 0, 0, 0, 0, -1], [-2, 0, 0, 0, 0, 1, 1, 0], [0, 0, -2, 1, 0, 0, 0, 0], [0, -2, 1, 1, 0, 0, 0, 1], [2, 0, 0, 0, 2, 1, 1, 0], [0, 1, 1, -2, 0, 0, 0, -3], [5, 0, 0, 0, -3, 3, 3, 0], [-6, 0, 0, 0, 2, -3, -1, 0], [0, 1, -2, -3, 0, 0, 0, 2], [-2, 0, 0, 0, -1, 3, -1, 0], [0, -1, 1, -5, 0, 0, 0, -2], [0, 0, 0, -5, 0, 0, 0, -1], [0, 3, -3, 1, 0, 0, 0, 3], [4, 0, 0, 0, 0, -2, -6, 0], [0, 0, 0, 0, 1, 0, 6, 0], [0, 6, 4, -5, 0, 0, 0, 1], [-8, 0, 0, 0, 2, -1, -1, 0], [0, 0, 2, -4, 0, 0, 0, -1], [4, 0, 0, 0, 0, -6, -2, 0], [0, 3, 1, 0, 0, 0, 0, 7], [5, 0, 0, 0, 3, -4, 2, 0], [0, 2, -3, 0, 0, 0, 0, -6], [-9, 0, 0, 0, -1, -9, -1, 0], [0, 0, 2, -7, 0, 0, 0, -1], [0, 5, 3, -4, 0, 0, 0, 1], [2, 0, 0, 0, -1, -6, 0, 0], [0, 6, 1, -11, 0, 0, 0, 3], [0, 15, 3, -5, 0, 0, 0, 2], [3, 0, 0, 0, -1, 1, 6, 0], [0, -3, -1, -2, 0, 0, 0, -10], [-3, 0, 0, 0, -5, -4, 3, 0], [5, 0, 0, 0, 6, 8, -1, 0], [-7, 0, 0, 0, -7, 0, -3, 0], [0, -5, -2, -1, 0, 0, 0, 4], [0, 5, 1, -2, 0, 0, 0, 7], [0, 4, 0, -8, 0, 0, 0, 0], [0, 7, 8, 0, 0, 0, 0, -5], [-3, 0, 0, 0, 1, -3, 0, 0], [-2, 0, 0, 0, -12, -2, -3, 0], [1, 0, 0, 0, -5, 5, 0, 0], [0, 6, -4, 2, 0, 0, 0, -7], [0, -1, -11, 5, 0, 0, 0, 1], [2, 0, 0, 0, -6, -5, 1, 0], [-6, 0, 0, 0, 2, -12, -6, 0], [0, -14, -4, 3, 0, 0, 0, -1], [0, -2, 2, 0, 0, 0, 0, 6], [-4, 0, 0, 0, 3, 2, -12, 0], [0, -3, 1, 2, 0, 0, 0, 8], [2, 0, 0, 0, -2, 2, 0, 0], [2, 0, 0, 0, 2, -5, 6, 0], [-1, 0, 0, 0, 5, 6, 5, 0], [0, -7, 1, -2, 0, 0, 0, 6], [0, -5, -9, 10, 0, 0, 0, -1], [-5, 0, 0, 0, 2, -2, 7, 0], [-5, 0, 0, 0, 7, -3, 0, 0], [0, -3, -2, -5, 0, 0, 0, -4], [-10, 0, 0, 0, 5, -1, 1, 0], [0, 2, -4, -3, 0, 0, 0, 7], [0, 7, 4, 8, 0, 0, 0, -3], [0, 9, -5, -3, 0, 0, 0, -10], [1, 0, 0, 0, 3, -3, -12, 0], [0, -14, 4, -6, 0, 0, 0, -8], [0, -4, 8, 14, 0, 0, 0, -2], [-5, 0, 0, 0, 5, -4, -3, 0], [0, 18, 8, -4, 0, 0, 0, 0], [0, -1, 1, 1, 0, 0, 0, 2], [-12, 0, 0, 0, -10, -4, 5, 0], [0, -4, 10, 6, 0, 0, 0, 2], [-3, 0, 0, 0, 5, 6, 1, 0], [0, 7, -11, 0, 0, 0, 0, 1], [0, 6, 6, -4, 0, 0, 0, 0], [12, 0, 0, 0, 4, -4, 0, 0], [0, 15, -5, -10, 0, 0, 0, 3], [1, 0, 0, 0, 6, 10, -7, 0], [0, 2, -2, -8, 0, 0, 0, -6], [6, 0, 0, 0, -5, 9, 9, 0], [-14, 0, 0, 0, -8, -1, 10, 0], [0, 0, 0, 0, 0, 0, 12, 0], [-8, 0, 0, 0, 12, -6, 7, 0], [2, 0, 0, 0, 0, 9, -9, 0], [0, 14, 0, 0, 0, 0, 0, -5], [-9, 0, 0, 0, -5, 3, 2, 0], [0, 18, -2, -5, 0, 0, 0, 3], [3, 0, 0, 0, 7, 4, 4, 0], [0, 12, 0, -2, 0, 0, 0, 4], [13, 0, 0, 0, 3, 11, 5, 0], [0, -5, -1, -7, 0, 0, 0, -4], [0, -28, 4, -1, 0, 0, 0, 1], [-11, 0, 0, 0, -7, 0, 3, 0], [0, -10, 0, -11, 0, 0, 0, 1], [0, 0, 0, 0, -4, -18, 2, 0], [19, 0, 0, 0, -3, 10, 3, 0], [0, -8, 0, 8, 0, 0, 0, -12], [5, 0, 0, 0, -1, 13, -5, 0], [12, 0, 0, 0, -5, 3, -5, 0], [0, 10, -4, -25, 0, 0, 0, -3], [-14, 0, 0, 0, 4, 12, 11, 0], [0, 9, 5, -4, 0, 0, 0, 13], [0, -19, 2, 4, 0, 0, 0, -1], [0, 8, 0, -12, 0, 0, 0, 8], [-13, 0, 0, 0, -7, -10, 2, 0], [5, 0, 0, 0, -5, 5, 10, 0], [0, 6, 8, 12, 0, 0, 0, 4], [0, 3, 5, -20, 0, 0, 0, 3], [0, 2, 5, 11, 0, 0, 0, 3], [-3, 0, 0, 0, 9, 6, 1, 0], [1, 0, 0, 0, -8, 1, -6, 0], [0, 11, 3, -15, 0, 0, 0, 12], [0, 31, 4, -21, 0, 0, 0, 4], [0, 5, -9, -10, 0, 0, 0, -4], [-26, 0, 0, 0, 10, -4, -6, 0], [12, 0, 0, 0, -8, -5, -1, 0], [-14, 0, 0, 0, 20, -8, 4, 0], [0, 0, -4, -12, 0, 0, 0, 0], [0, -17, 11, 12, 0, 0, 0, 9], [0, 19, 2, -24, 0, 0, 0, 1], [-7, 0, 0, 0, 11, -8, -3, 0], [-10, 0, 0, 0, 8, -18, 8, 0], [0, 10, -9, 8, 0, 0, 0, 16], [0, 10, -18, -2, 0, 0, 0, -2], [0, -2, -6, -2, 0, 0, 0, -8], [17, 0, 0, 0, 7, 21, -2, 0], [19, 0, 0, 0, 0, 0, -3, 0], [10, 0, 0, 0, 1, 6, -4, 0], [13, 0, 0, 0, 1, 26, 1, 0], [0, 14, -8, 1, 0, 0, 0, -3], [0, 6, 0, -28, 0, 0, 0, -6], [4, 0, 0, 0, 4, -1, 21, 0], [0, -6, -2, 5, 0, 0, 0, -1], [-14, 0, 0, 0, -10, -13, -3, 0], [0, -12, 10, -7, 0, 0, 0, -18], [8, 0, 0, 0, 7, -5, 15, 0], [6, 0, 0, 0, -8, -4, 2, 0], [0, 1, -13, 9, 0, 0, 0, 1], [0, -5, 5, 17, 0, 0, 0, -12], [0, -21, 11, 17, 0, 0, 0, -1], [-11, 0, 0, 0, -19, 0, 2, 0], [-3, 0, 0, 0, 11, 7, 8, 0], [20, 0, 0, 0, 0, 34, 2, 0], [0, -10, -16, 5, 0, 0, 0, 9], [0, 7, -7, -5, 0, 0, 0, -4], [6, 0, 0, 0, 5, -22, 20, 0], [-13, 0, 0, 0, 1, -13, 0, 0], [0, -26, -10, 19, 0, 0, 0, -10], [0, 16, -4, 12, 0, 0, 0, 6], [-16, 0, 0, 0, 20, 0, 4, 0], [0, 12, 4, -27, 0, 0, 0, 1], [0, 6, -2, -13, 0, 0, 0, 2], [18, 0, 0, 0, 10, 14, -8, 0], [0, -13, 1, -1, 0, 0, 0, -10], [0, 19, 3, -23, 0, 0, 0, 14], [0, 9, -21, -15, 0, 0, 0, 2], [8, 0, 0, 0, -10, -3, 9, 0], [-17, 0, 0, 0, 7, 4, -13, 0], [3, 0, 0, 0, -5, 24, 0, 0], [0, 8, -11, -4, 0, 0, 0, 4], [31, 0, 0, 0, -11, 31, 7, 0], [0, 37, 11, -12, 0, 0, 0, 1], [0, -4, 9, 21, 0, 0, 0, 3], [0, -6, 2, 14, 0, 0, 0, -8], [17, 0, 0, 0, 7, -4, 1, 0], [0, -5, 3, -4, 0, 0, 0, -2], [0, -7, 13, 7, 0, 0, 0, -2], [-51, 0, 0, 0, 1, -5, 0, 0], [0, 6, 8, 0, 0, 0, 0, 6], [-13, 0, 0, 0, 10, -7, -10, 0], [0, 15, 9, -7, 0, 0, 0, 1], [-3, 0, 0, 0, 3, -10, -21, 0], [-24, 0, 0, 0, -5, -26, -2, 0], [2, 0, 0, 0, 0, -19, 3, 0], [0, 18, 9, -6, 0, 0, 0, -6], [23, 0, 0, 0, -5, 18, 9, 0], [-11, 0, 0, 0, 9, -6, -12, 0], [8, 0, 0, 0, 8, 15, 17, 0], [-1, 0, 0, 0, -3, 19, -1, 0], [0, -26, -2, 8, 0, 0, 0, 10], [0, 0, 0, 0, -6, -12, 19, 0], [0, 36, 2, -1, 0, 0, 0, 1], [1, 0, 0, 0, 3, -14, -21, 0], [0, 19, 4, -3, 0, 0, 0, 20], [0, 22, 9, -17, 0, 0, 0, -11], [0, -9, 11, -2, 0, 0, 0, 15], [30, 0, 0, 0, -4, 28, 12, 0], [0, -1, 18, 1, 0, 0, 0, 0], [0, -8, 6, -15, 0, 0, 0, 17], [-2, 0, 0, 0, -8, 3, -10, 0], [35, 0, 0, 0, 7, -2, -1, 0], [0, -38, -12, 30, 0, 0, 0, -4], [0, -32, -8, 5, 0, 0, 0, -1], [-13, 0, 0, 0, 1, 5, 8, 0], [-5, 0, 0, 0, -2, 0, 15, 0], [0, 18, 2, 12, 0, 0, 0, -6], [0, 19, -1, -30, 0, 0, 0, 0], [-2, 0, 0, 0, -14, 16, -2, 0], [0, -24, -8, 25, 0, 0, 0, -4], [0, -15, -11, 30, 0, 0, 0, 0], [0, 2, 18, 9, 0, 0, 0, -5], [7, 0, 0, 0, -4, 0, 17, 0], [19, 0, 0, 0, 3, 42, 5, 0], [0, 2, 10, -8, 0, 0, 0, 12], [21, 0, 0, 0, 6, 15, -20, 0], [-25, 0, 0, 0, 1, -10, 13, 0], [0, -37, 0, 14, 0, 0, 0, -3], [38, 0, 0, 0, -6, 16, -6, 0], [0, -17, 9, -7, 0, 0, 0, 10], [24, 0, 0, 0, 11, 13, 5, 0], [-17, 0, 0, 0, 9, 2, -1, 0], [0, -12, -8, 30, 0, 0, 0, 1], [-14, 0, 0, 0, -4, 8, -24, 0], [0, 2, 4, -7, 0, 0, 0, 1], [0, -20, 16, 20, 0, 0, 0, 20], [-14, 0, 0, 0, 16, -19, 3, 0], [-17, 0, 0, 0, -12, -1, -6, 0], [0, -1, 15, 7, 0, 0, 0, -2], [12, 0, 0, 0, -9, 1, -5, 0], [0, 20, -4, 8, 0, 0, 0, 16], [0, 29, -6, 0, 0, 0, 0, -11], [-4, 0, 0, 0, 20, -8, 17, 0], [28, 0, 0, 0, 12, 11, -1, 0], [10, 0, 0, 0, -10, -10, -12, 0], [0, -23, -9, 20, 0, 0, 0, -21], [0, -6, -2, -4, 0, 0, 0, -22], [-24, 0, 0, 0, 18, -24, 1, 0], [0, 1, -19, -4, 0, 0, 0, 16], [4, 0, 0, 0, -18, 3, 19, 0], [0, 4, -2, -3, 0, 0, 0, -9], [-16, 0, 0, 0, 4, -18, 10, 0], [0, 25, 16, -9, 0, 0, 0, -10], [19, 0, 0, 0, -7, -17, -10, 0], [-10, 0, 0, 0, 6, 12, 18, 0], [1, 0, 0, 0, 7, 26, -2, 0], [0, -17, -5, 0, 0, 0, 0, -1], [0, -3, -15, 8, 0, 0, 0, -9], [-18, 0, 0, 0, 8, -21, 4, 0], [0, -7, -1, 37, 0, 0, 0, -1], [-1, 0, 0, 0, 5, -16, 31, 0], [-7, 0, 0, 0, -11, -28, 3, 0], [0, -29, -3, 31, 0, 0, 0, -12], [-41, 0, 0, 0, 1, -32, -7, 0], [0, 22, 10, 20, 0, 0, 0, -6], [10, 0, 0, 0, 5, -34, -2, 0], [0, 25, 13, 18, 0, 0, 0, -16], [-18, 0, 0, 0, 32, 3, 5, 0], [0, -16, -4, 8, 0, 0, 0, 20], [12, 0, 0, 0, 18, 1, -1, 0], [16, 0, 0, 0, 0, -9, -7, 0], [0, -24, -4, 1, 0, 0, 0, 13], [0, -27, -22, 15, 0, 0, 0, -2], [-27, 0, 0, 0, -5, 4, 2, 0], [0, 32, -12, -25, 0, 0, 0, -13], [7, 0, 0, 0, -20, 21, 10, 0], [0, 24, -8, 10, 0, 0, 0, 14], [-36, 0, 0, 0, 10, -3, -17, 0], [14, 0, 0, 0, -4, 16, -5, 0], [0, 14, 14, -24, 0, 0, 0, -6], [0, -23, 4, -10, 0, 0, 0, 15], [0, -18, 9, 40, 0, 0, 0, -4], [0, 34, 8, -3, 0, 0, 0, 13], [0, 21, -9, -6, 0, 0, 0, 19], [-26, 0, 0, 0, -17, -8, 8, 0], [0, -2, 4, 27, 0, 0, 0, -18], [0, -4, 5, -3, 0, 0, 0, -11], [-30, 0, 0, 0, 2, 18, 0, 0], [25, 0, 0, 0, 16, -20, 25, 0], [21, 0, 0, 0, -19, 18, 4, 0], [0, -14, 16, 25, 0, 0, 0, 3], [0, 19, -25, -11, 0, 0, 0, -13], [-28, 0, 0, 0, 7, -28, -14, 0], [0, -27, -27, 14, 0, 0, 0, -9], [0, 22, -14, -12, 0, 0, 0, -7], [7, 0, 0, 0, -17, -11, 12, 0], [0, -46, 15, 34, 0, 0, 0, 10], [0, 14, -26, 6, 0, 0, 0, 0], [0, -1, -11, -20, 0, 0, 0, 3], [-2, 0, 0, 0, 2, 2, 3, 0], [12, 0, 0, 0, -12, 3, 26, 0], [-2, 0, 0, 0, -30, -14, 4, 0], [0, 29, 13, -42, 0, 0, 0, -3], [-18, 0, 0, 0, 14, 24, 10, 0], [0, 19, -9, 38, 0, 0, 0, 11], [32, 0, 0, 0, 3, 14, 10, 0], [0, -5, 31, 12, 0, 0, 0, -5], [-16, 0, 0, 0, -6, -35, 5, 0], [0, -14, -2, 10, 0, 0, 0, -17], [0, -1, -6, -24, 0, 0, 0, 15], [-19, 0, 0, 0, -19, -27, 10, 0], [12, 0, 0, 0, -1, -19, -25, 0], [-59, 0, 0, 0, 9, -13, -11, 0], [0, -7, 11, -1, 0, 0, 0, -18], [0, -2, -7, 11, 0, 0, 0, 5], [-42, 0, 0, 0, 12, -29, -21, 0], [0, 42, 8, -1, 0, 0, 0, 2], [-1, 0, 0, 0, 2, 4, 15, 0], [23, 0, 0, 0, 31, 19, -14, 0], [0, -11, 20, -2, 0, 0, 0, -5], [6, 0, 0, 0, 22, -8, -18, 0], [0, 11, 5, 7, 0, 0, 0, -6], [0, -34, 4, 18, 0, 0, 0, -7], [0, 7, -6, -20, 0, 0, 0, 7], [27, 0, 0, 0, 7, 14, -11, 0]]
